# -*- coding: utf-8 -*-
#
#    Copyright 2019-2020 Fabien Bourgeois <fabien@yaltik.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" Odoo XML DSL """

from .xml_base import xmlroot, xmln

# XML helpers functions and macros

def odoo(*args): return xmlroot(xmln('odoo', {}, *args))

def data(*args):
    if len(args) == 1:
        return xmln('data', {}, *args)
    return xmln('data', *args)

def function(*args): return xmln('function', *args)
def record(*args): return xmln('record', *args)
def form(*args): return xmln('form', *args)
def tree(*args): return xmln('tree', *args)
def search(*args): return xmln('search', *args)

def act_window(*args): return xmln('act_window', *args)

def act_window_model(model, attrs):
    """ Build new act_window from model and args """
    model_und = model.replace('.', '_')
    model_cap = ' '.join([w.capitalize() for w in model.split('.')])
    xmlid = '{:{}}'.format(model_und, '') + '_view_action'
    name = '{:{}}'.format(model_cap, '') + ' Action'
    attrs.update({'id': xmlid, 'name': name, 'res_model': model})
    return act_window(attrs)

def menuitem(*args): return xmln('menuitem', *args)

def menuitem_model(model, attrs):
    """ Build new menuitem from model and attrs """
    model_und = model.replace('.', '_')
    actionid = '{:{}}'.format(model_und, '') + '_view_action'
    xmlid = '{:{}}'.format(model_und, '') + '_men'
    attrs.update({'id': xmlid, 'action': actionid})
    return menuitem(attrs)


def group(*args): return xmln('group', *args)
def header(*args): return xmln('header', *args)
def footer(*args): return xmln('footer', *args)
def sheet(*args): return xmln('sheet', *args)
def button(*args): return xmln('button', *args)
def p(*args): return xmln('p', *args)
def xpath(*args): return xmln('xpath', *args)
def attribute(name, value): return xmln('attribute', {'name': name}, [value])

def field(*args): return xmln('field', *args)
def field_name(name): return field({'name': 'name'}, [name])
def field_model(model): return field({'name': 'model'}, [model])
def field_inherit(xmlid): return field({'name': 'inherit_id', 'ref': xmlid}, [])
def field_arch(*args): return field({'name': 'arch', 'type': 'xml'}, *args)

def view(xmlid, children): return record({'id': xmlid, 'model': 'ir.ui.view'}, children)

def view_def(xmlid, name, model, arch):
    """ View and first fields simplification with record xmlid, name, targeted model """
    return view(xmlid, [field_name(name), field_model(model), field_arch(arch)])

def view_new(view_type, model, arch):
    """ View : new view definition, based on type (form, tree, ...) and model ID """
    model_und = model.replace('.', '_')
    model_cap = ' '.join([w.capitalize() for w in model.split('.')])
    xmlid = '{:{}}'.format(model_und, '') + '_view_' + '{:{}}'.format(
        view_type, '')
    name = '{:{}}'.format(model_cap, '') + ' ' + '{:{}}'.format(view_type.
        capitalize(), '')
    return view_def(xmlid=xmlid, name=name, model=model, arch=arch)

def view_inherit(filename, model, inherit, arch):
    """ Inherited View simplification with name of the record, xmlid for model
    and inherited view """
    module = filename.split('.')[2]
    inherited = inherit.split('.')[1]
    xmlid = '{:{}}'.format(inherited, '') + '_inherit_' + '{:{}}'.format(
        module, '')
    model_cap = ' '.join([w.capitalize() for w in model.split('.')])
    name = '{:{}}'.format(model_cap, '') + ' Adaptations'
    return view(xmlid, [field_name(name), field_model(model), field_inherit
        (inherit), field_arch(arch)])

def actions_server_code(xmlid, name, modelref, code):
    """ Server actions of type code """
    return record({'id': xmlid, 'model': 'ir.actions.server'}, [
        field_name(name), field({'name': 'model_id', 'ref': modelref}, [
        ]), field({'name': 'state'}, ['code']), field({'name': 'code'},
        [code])])

def client_action_multi(xmlid, name, model, action):
    """ Client action multi (ir.values), with own xmlid, name, targeted model
    and  action """
    action = u"'ir.actions.server,%d'%" + '{:{}}'.format(action, '')
    return record({'id': xmlid, 'model': 'ir.values'}, [field_name(name),
        field({'name': 'key2', 'eval': u"'client_action_multi'"}, []),
        field({'name': 'model', 'eval': u"'" + model + u"'"}, []), field
        ({'name': 'value', 'eval': action})])
